@@ -199,7 +199,7 @@ static void dopr (char *buffer, size_t maxlen, const char *format, va_list args)
       }
       break;
     case DP_S_MIN:
-      if (isdigit(ch)) 
+      if (isdigit((int) ch)) 
       {
 	min = 10*min + char_to_int (ch);
 	ch = *format++;
@@ -223,7 +223,7 @@ static void dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 	state = DP_S_MOD;
       break;
     case DP_S_MAX:
-      if (isdigit(ch)) 
+      if (isdigit((int) ch)) 
       {
 	if (max < 0)
 	  max = 0;
@@ -399,7 +399,7 @@ static void fmtstr (char *buffer, size_t *currlen, size_t maxlen,
   
   if (value == 0)
   {
-    value = "<NULL>";
+    value = (char *) "<NULL>";
   }
 
   for (strln = 0; value[strln]; ++strln); /* strlen */
@@ -526,7 +526,7 @@ static long double abs_val (long double value)
   return result;
 }
 
-static long double pow10 (int exp)
+static double pow10 (double exp)
 {
   long double result = 1;
 
@@ -539,7 +539,7 @@ static long double pow10 (int exp)
   return result;
 }
 
-static long round (long double value)
+static double round (double value)
 {
   long intpart;
 
