@@ -141,6 +141,14 @@ ldns_sign_public_buffer(ldns_buffer *sign_buf, ldns_key *current_key)
 				   EVP_sha512());
 		break;
 #endif /* USE_SHA2 */
+#ifdef USE_GOST
+	case LDNS_SIGN_GOST:
+		b64rdf = ldns_sign_public_evp(
+				   sign_buf,
+				   ldns_key_evp_key(current_key),
+				   EVP_get_digestbyname("md_gost94"));
+		break;
+#endif /* USE_GOST */
 	case LDNS_SIGN_RSAMD5:
 		b64rdf = ldns_sign_public_evp(
 				   sign_buf,
