@@ -508,6 +508,8 @@ ldns_dnssec_trust_tree_print_sm(FILE *out,
 				} else if (ldns_rr_get_type(tree->rr) == LDNS_RR_TYPE_DS) {
 					fprintf(out, " keytag: ");
 					ldns_rdf_print(out, ldns_rr_rdf(tree->rr, 0));
+					fprintf(out, " digest type: ");
+					ldns_rdf_print(out, ldns_rr_rdf(tree->rr, 2));
 				}
 				if (ldns_rr_get_type(tree->rr) == LDNS_RR_TYPE_NSEC) {
 					fprintf(out, " ");
@@ -978,15 +980,12 @@ ldns_verify(ldns_rr_list *rrset, ldns_rr_list *rrsig, const ldns_rr_list *keys,
 		  ldns_rr_list *good_keys)
 {
 	uint16_t i;
-	bool valid;
 	ldns_status verify_result = LDNS_STATUS_ERR;
 
 	if (!rrset || !rrsig || !keys) {
 		return LDNS_STATUS_ERR;
 	}
 
-	valid = false;
-
 	if (ldns_rr_list_rr_count(rrset) < 1) {
 		return LDNS_STATUS_ERR;
 	}
@@ -1014,6 +1013,44 @@ ldns_verify(ldns_rr_list *rrset, ldns_rr_list *rrsig, const ldns_rr_list *keys,
 	return verify_result;
 }
 
+ldns_status
+ldns_verify_notime(ldns_rr_list *rrset, ldns_rr_list *rrsig, 
+	const ldns_rr_list *keys, ldns_rr_list *good_keys)
+{
+	uint16_t i;
+	ldns_status verify_result = LDNS_STATUS_ERR;
+
+	if (!rrset || !rrsig || !keys) {
+		return LDNS_STATUS_ERR;
+	}
+
+	if (ldns_rr_list_rr_count(rrset) < 1) {
+		return LDNS_STATUS_ERR;
+	}
+
+	if (ldns_rr_list_rr_count(rrsig) < 1) {
+		return LDNS_STATUS_CRYPTO_NO_RRSIG;
+	}
+	
+	if (ldns_rr_list_rr_count(keys) < 1) {
+		verify_result = LDNS_STATUS_CRYPTO_NO_TRUSTED_DNSKEY;
+	} else {
+		for (i = 0; i < ldns_rr_list_rr_count(rrsig); i++) {
+			ldns_status s = ldns_verify_rrsig_keylist_notime(rrset, 
+				ldns_rr_list_rr(rrsig, i), keys, good_keys);
+			/* try a little to get more descriptive error */
+			if(s == LDNS_STATUS_OK) {
+				verify_result = LDNS_STATUS_OK;
+			} else if(verify_result == LDNS_STATUS_ERR)
+				verify_result = s;
+			else if(s !=  LDNS_STATUS_ERR && verify_result ==
+				LDNS_STATUS_CRYPTO_NO_MATCHING_KEYTAG_DNSKEY)
+				verify_result = s;
+		}
+	}
+	return verify_result;
+}
+
 ldns_rr_list *
 ldns_fetch_valid_domain_keys(const ldns_resolver *res,
                              const ldns_rdf *domain,
@@ -1473,6 +1510,54 @@ ldns_dnssec_verify_denial_nsec3(ldns_rr *rr,
 }
 #endif /* HAVE_SSL */
 
+#ifdef USE_GOST
+EVP_PKEY*
+ldns_gost2pkey_raw(unsigned char* key, size_t keylen)
+{
+	/* prefix header for X509 encoding */
+	uint8_t asn[37] = { 0x30, 0x63, 0x30, 0x1c, 0x06, 0x06, 0x2a, 0x85, 
+		0x03, 0x02, 0x02, 0x13, 0x30, 0x12, 0x06, 0x07, 0x2a, 0x85, 
+		0x03, 0x02, 0x02, 0x23, 0x01, 0x06, 0x07, 0x2a, 0x85, 0x03, 
+		0x02, 0x02, 0x1e, 0x01, 0x03, 0x43, 0x00, 0x04, 0x40};
+	unsigned char encoded[37+64];
+	const unsigned char* pp;
+	if(keylen != 64) {
+		/* key wrong size */
+		return NULL;
+	}
+
+	/* create evp_key */
+	memmove(encoded, asn, 37);
+	memmove(encoded+37, key, 64);
+	pp = (unsigned char*)&encoded[0];
+
+	return d2i_PUBKEY(NULL, &pp, sizeof(encoded));
+}
+
+static ldns_status
+ldns_verify_rrsig_gost_raw(unsigned char* sig, size_t siglen, 
+	ldns_buffer* rrset, unsigned char* key, size_t keylen)
+{
+	EVP_PKEY *evp_key;
+	ldns_status result;
+
+	(void) ldns_key_EVP_load_gost_id();
+	evp_key = ldns_gost2pkey_raw(key, keylen);
+	if(!evp_key) {
+		/* could not convert key */
+		return LDNS_STATUS_CRYPTO_BOGUS;
+	}
+
+	/* verify signature */
+	result = ldns_verify_rrsig_evp_raw(sig, siglen, rrset, 
+		evp_key, EVP_get_digestbyname("md_gost94"));
+	EVP_PKEY_free(evp_key);
+
+	return result;
+}
+#endif
+
+
 ldns_status
 ldns_verify_rrsig_buffers(ldns_buffer *rawsig_buf, ldns_buffer *verify_buf, 
 					 ldns_buffer *key_buf, uint8_t algo)
@@ -1523,6 +1608,12 @@ ldns_verify_rrsig_buffers_raw(unsigned char* sig, size_t siglen,
 									    key,
 									    keylen);
 		break;
+#endif
+#ifdef USE_GOST
+	case LDNS_GOST:
+		return ldns_verify_rrsig_gost_raw(sig, siglen, verify_buf,
+			key, keylen);
+		break;
 #endif
 	case LDNS_RSAMD5:
 		return ldns_verify_rrsig_rsamd5_raw(sig,
@@ -1608,6 +1699,9 @@ ldns_rrsig2rawsig_buffer(ldns_buffer* rawsig_buf, ldns_rr* rrsig)
 #ifdef USE_SHA2
 	case LDNS_RSASHA256:
 	case LDNS_RSASHA512:
+#endif
+#ifdef USE_GOST
+	case LDNS_GOST:
 #endif
 		if (ldns_rdf2buffer_wire(rawsig_buf, 
 		    ldns_rr_rdf(rrsig, 8)) != LDNS_STATUS_OK) {
@@ -1776,6 +1870,35 @@ ldns_verify_rrsig_keylist(ldns_rr_list *rrset,
 					 ldns_rr *rrsig,
 					 const ldns_rr_list *keys, 
 					 ldns_rr_list *good_keys)
+{
+	ldns_status result;
+	ldns_rr_list *valid = ldns_rr_list_new();
+	if (!valid)
+		return LDNS_STATUS_MEM_ERR;
+
+	result = ldns_verify_rrsig_keylist_notime(rrset, rrsig, keys, valid);
+	if(result != LDNS_STATUS_OK) {
+		ldns_rr_list_free(valid); 
+		return result;
+	}
+
+	/* check timestamps last; its OK except time */
+	result = ldns_rrsig_check_timestamps(rrsig, (int32_t)time(NULL));
+	if(result != LDNS_STATUS_OK) {
+		ldns_rr_list_free(valid); 
+		return result;
+	}
+
+	ldns_rr_list_cat(good_keys, valid);
+	ldns_rr_list_free(valid);
+	return LDNS_STATUS_OK;
+}
+
+ldns_status
+ldns_verify_rrsig_keylist_notime(ldns_rr_list *rrset,
+					 ldns_rr *rrsig,
+					 const ldns_rr_list *keys, 
+					 ldns_rr_list *good_keys)
 {
 	ldns_buffer *rawsig_buf;
 	ldns_buffer *verify_buf;
@@ -1845,12 +1968,7 @@ ldns_verify_rrsig_keylist(ldns_rr_list *rrset,
 		return result;
 	}
 
-	/* check timestamps last; its OK except time */
-	result = ldns_rrsig_check_timestamps(rrsig, (int32_t)time(NULL));
-	if(result != LDNS_STATUS_OK) {
-		ldns_rr_list_free(validkeys); 
-		return result;
-	}
+	/* do not check timestamps */
 
 	ldns_rr_list_cat(good_keys, validkeys);
 	ldns_rr_list_free(validkeys);
@@ -2074,7 +2192,6 @@ ldns_verify_rrsig_rsasha512_raw(unsigned char* sig,
 }
 
 
-
 ldns_status
 ldns_verify_rrsig_rsamd5_raw(unsigned char* sig,
 					    size_t siglen,
