@@ -8,14 +8,16 @@
  * send a notify packet to a server
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdbool.h>
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <errno.h>
 
 /* ldns */
-#include "ldns/ldns.h"
+#include <ldns/ldns.h>
 
 static int verbose = 1;
 static int max_num_retry = 15; /* times to try */
@@ -247,6 +249,7 @@ main(int argc, char **argv)
 	}
 
 	if(tsig_cred.keyname) {
+#ifdef HAVE_SSL
 		status = ldns_pkt_tsig_sign(notify, tsig_cred.keyname,
 			tsig_cred.keydata, 300, tsig_cred.algorithm,
 			NULL);
@@ -254,6 +257,9 @@ main(int argc, char **argv)
 			printf("Error TSIG sign query: %s\n",
 				ldns_get_errorstr_by_id(status));
 		}
+#else
+	fprintf(stderr, "Warning: TSIG needs OpenSSL support, which has not been compiled in, TSIG skipped\n");
+#endif
 	}
 
 	if(verbose) {
