@@ -143,7 +143,9 @@ struct sockaddr_storage;
 #ifdef HAVE_NETINET_UDP_H
 #include <netinet/udp.h>
 #endif
+#ifdef HAVE_NETINET_IGMP_H
 #include <netinet/igmp.h>
+#endif
 #include <errno.h>
 
 #define INBUF_SIZE 4096         /* max size for incoming queries */
@@ -249,7 +251,14 @@ handle_udp(int udp_sock, struct entry* entries, int *count)
 	nb = recvfrom(udp_sock, inbuf, INBUF_SIZE, 0, 
 		(struct sockaddr*)&userdata.addr_him, &userdata.hislen);
 	if (nb < 1) {
+#ifndef USE_WINSOCK
 		log_msg("recvfrom(): %s\n", strerror(errno));
+#else
+		if(WSAGetLastError() != WSAEINPROGRESS &&
+			WSAGetLastError() != WSAECONNRESET &&
+			WSAGetLastError()!= WSAEWOULDBLOCK)
+			log_msg("recvfrom(): %d\n", WSAGetLastError());
+#endif
 		return;
 	}
 	handle_query(inbuf, nb, entries, count, transport_udp, send_udp, 
@@ -261,9 +270,9 @@ read_n_bytes(int sock, uint8_t* buf, size_t sz)
 {
 	size_t count = 0;
 	while(count < sz) {
-		ssize_t nb = read(sock, buf+count, sz-count);
+		ssize_t nb = recv(sock, buf+count, sz-count, 0);
 		if(nb < 0) {
-			log_msg("read(): %s\n", strerror(errno));
+			log_msg("recv(): %s\n", strerror(errno));
 			return;
 		}
 		count += nb;
@@ -275,9 +284,9 @@ write_n_bytes(int sock, uint8_t* buf, size_t sz)
 {
 	size_t count = 0;
 	while(count < sz) {
-		ssize_t nb = write(sock, buf+count, sz-count);
+		ssize_t nb = send(sock, buf+count, sz-count, 0);
 		if(nb < 0) {
-			log_msg("write(): %s\n", strerror(errno));
+			log_msg("send(): %s\n", strerror(errno));
 			return;
 		}
 		count += nb;
@@ -333,12 +342,69 @@ handle_tcp(int tcp_sock, struct entry* entries, int *count)
 
 }
 
+/** shared by the service and main routine (forked and threaded) */
+static int udp_sock, tcp_sock;
+static struct entry* entries;
+
+/** 
+ * Test DNS server service, uses global udpsock, tcpsock, reply entries 
+ * The signature is kept void so the function can be used as a thread function.
+ */
+static void
+service(void)
+{
+	fd_set rset, wset, eset;
+	struct timeval timeout;
+	int count;
+	int maxfd;
+
+	/* service */
+	count = 0;
+	timeout.tv_sec = 0;
+	timeout.tv_usec = 0;
+	while (1) {
+		FD_ZERO(&rset);
+		FD_ZERO(&wset);
+		FD_ZERO(&eset);
+		FD_SET(udp_sock, &rset);
+		FD_SET(tcp_sock, &rset);
+		maxfd = udp_sock;
+		if(tcp_sock > maxfd)
+			maxfd = tcp_sock;
+		if(select(maxfd+1, &rset, &wset, &eset, NULL) < 0) {
+			error("select(): %s\n", strerror(errno));
+		}
+		if(FD_ISSET(udp_sock, &rset)) {
+			handle_udp(udp_sock, entries, &count);
+		}
+		if(FD_ISSET(tcp_sock, &rset)) {
+			handle_tcp(tcp_sock, entries, &count);
+		}
+	}
+}
+
 static void
 forkit(int number)
 {
 	int i;
 	for(i=0; i<number; i++)
 	{
+#ifndef HAVE_FORK
+#ifndef USE_WINSOCK
+		log_msg("fork() not available.\n");
+		exit(1);
+#else /* USE_WINSOCK */
+		DWORD tid;
+		HANDLE id = CreateThread(NULL, 0, 
+			(LPTHREAD_START_ROUTINE)service, NULL,
+			0, &tid);
+		if(id == NULL) {
+			log_msg("error CreateThread: %d\n", GetLastError());
+			return;
+		}
+		log_msg("thread id: %d\n", (int)tid);
+#endif /* USE_WINSOCK */
+#else /* HAVE_FORK */
 		pid_t pid = fork();
 		if(pid == -1) {
 			log_msg("error forking: %s\n", strerror(errno));
@@ -347,6 +413,7 @@ forkit(int number)
 		if(pid == 0)
 			return; /* child starts serving */
 		log_msg("forked pid: %d\n", (int)pid);
+#endif /* HAVE_FORK */
 	}
 }
 
@@ -357,19 +424,15 @@ main(int argc, char **argv)
 	int c;
 	int port = DEFAULT_PORT;
 	const char* datafile;
-	int count;
 	int forknum = 0;
 
 	/* network */
 	int fam = AF_INET;
-	int udp_sock, tcp_sock;
-	fd_set rset, wset, eset;
-	struct timeval timeout;
-	int maxfd;
 	bool random_port_success;
 
-	/* dns */
-	struct entry* entries;
+#ifdef USE_WINSOCK
+	WSADATA wsa_data;
+#endif
 	
 	/* parse arguments */
 	srandom(time(NULL) ^ getpid());
@@ -417,6 +480,11 @@ main(int argc, char **argv)
 	datafile = argv[0];
 	log_msg("Reading datafile %s\n", datafile);
 	entries = read_datafile(datafile);
+
+#ifdef USE_WINSOCK
+	if(WSAStartup(MAKEWORD(2,2), &wsa_data) != 0)
+		error("WSAStartup failed\n");
+#endif
 	
 	if((udp_sock = socket(fam, SOCK_DGRAM, 0)) < 0) {
 		error("udp socket(): %s\n", strerror(errno));
@@ -425,7 +493,7 @@ main(int argc, char **argv)
 		error("tcp socket(): %s\n", strerror(errno));
 	}
 	c = 1;
-	if(setsockopt(tcp_sock, SOL_SOCKET, SO_REUSEADDR, &c, (socklen_t) sizeof(int)) < 0) {
+	if(setsockopt(tcp_sock, SOL_SOCKET, SO_REUSEADDR, (void*)&c, (socklen_t) sizeof(int)) < 0) {
 		error("setsockopt(SO_REUSEADDR): %s\n", strerror(errno));
 	}
 
@@ -447,7 +515,13 @@ main(int argc, char **argv)
 			log_msg("trying to bind to port %d\n", port);
 			random_port_success = true;
 			if (bind_port(udp_sock, port, fam)) {
+#ifdef EADDRINUSE
 				if (errno != EADDRINUSE) {
+#elif defined(USE_WINSOCK)
+				if (WSAGetLastError() != WSAEADDRINUSE) {
+#else
+				if (1) {
+#endif
 					perror("bind()");
 					return -1;
 				} else {
@@ -456,7 +530,13 @@ main(int argc, char **argv)
 			}
 			if (random_port_success) {
 				if (bind_port(tcp_sock, port, fam)) {
+#ifdef EADDRINUSE
 					if (errno != EADDRINUSE) {
+#elif defined(USE_WINSOCK)
+					if (WSAGetLastError()!=WSAEADDRINUSE){
+#else
+					if (1) {
+#endif
 						perror("bind()");
 						return -1;
 					} else {
@@ -478,28 +558,7 @@ main(int argc, char **argv)
 	if(forknum > 0)
 		forkit(forknum);
 
-	/* service */
-	count = 0;
-	timeout.tv_sec = 0;
-	timeout.tv_usec = 0;
-	while (1) {
-		FD_ZERO(&rset);
-		FD_ZERO(&wset);
-		FD_ZERO(&eset);
-		FD_SET(udp_sock, &rset);
-		FD_SET(tcp_sock, &rset);
-		maxfd = udp_sock;
-		if(tcp_sock > maxfd)
-			maxfd = tcp_sock;
-		if(select(maxfd+1, &rset, &wset, &eset, NULL) < 0) {
-			error("select(): %s\n", strerror(errno));
-		}
-		if(FD_ISSET(udp_sock, &rset)) {
-			handle_udp(udp_sock, entries, &count);
-		}
-		if(FD_ISSET(tcp_sock, &rset)) {
-			handle_tcp(tcp_sock, entries, &count);
-		}
-	}
+	service();
+
         return 0;
 }
