@@ -10,12 +10,21 @@
 #include "config.h"
 #include <ldns/ldns.h>
 
-#include <sys/socket.h>
-
-#include <arpa/inet.h>
-#include <netinet/in.h>
-#include <netinet/udp.h>
-#include <netinet/igmp.h>
+#ifdef HAVE_SYS_SOCKET_H
+#  include <sys/socket.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
+#  include <arpa/inet.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+#  include <netinet/in.h>
+#endif
+#ifdef HAVE_NETINET_UDP_H
+#  include <netinet/udp.h>
+#endif
+#ifdef HAVE_NETINET_IGMP_H
+#  include <netinet/igmp.h>
+#endif
 
 #include <errno.h>
 
@@ -87,10 +96,10 @@ main(int argc, char **argv)
 
 	/* network */
 	int sock;
-	size_t nb;
+	ssize_t nb;
 	struct sockaddr addr_me;
 	struct sockaddr addr_him;
-	socklen_t hislen;
+	socklen_t hislen = (socklen_t) sizeof(addr_him);
 	uint8_t inbuf[INBUF_SIZE];
 	uint8_t *outbuf;
 
@@ -104,6 +113,7 @@ main(int argc, char **argv)
 	ldns_rr_list *answer_an;
 	ldns_rr_list *answer_ns;
 	ldns_rr_list *answer_ad;
+	ldns_rdf *origin = NULL;
 	
 	/* zone */
 	ldns_zone *zone;
@@ -114,13 +124,18 @@ main(int argc, char **argv)
 	char *my_address = NULL;
 		
 	if (argc < 5) {
-		usage(stdout);
+		usage(stderr);
 		exit(EXIT_FAILURE);
 	} else {
-	        my_address = argv[1];
+	    my_address = argv[1];
 		port = atoi(argv[2]);
 		if (port < 1) {
-			usage(stdout);
+			usage(stderr);
+			exit(EXIT_FAILURE);
+		}
+		if (ldns_str2rdf_dname(&origin, argv[3]) != LDNS_STATUS_OK) {
+			fprintf(stderr, "Bad origin, not a correct domain name\n");
+			usage(stderr);
 			exit(EXIT_FAILURE);
 		}
 		zone_file = argv[4];
@@ -134,7 +149,7 @@ main(int argc, char **argv)
 	}
 	
 	line_nr = 0;
-	status = ldns_zone_new_frm_fp_l(&zone, zone_fp, NULL, 0, LDNS_RR_CLASS_IN, &line_nr);
+	status = ldns_zone_new_frm_fp_l(&zone, zone_fp, origin, 0, LDNS_RR_CLASS_IN, &line_nr);
 
 	if (status != LDNS_STATUS_OK) {
 		printf("Zone reader failed, aborting\n");
@@ -160,7 +175,7 @@ main(int argc, char **argv)
 
 	/* Done. Now receive */
 	while (1) {
-		nb = (size_t) recvfrom(sock, inbuf, INBUF_SIZE, 0, &addr_him, &hislen);
+		nb = recvfrom(sock, inbuf, INBUF_SIZE, 0, &addr_him, &hislen);
 		if (nb < 1) {
 			fprintf(stderr, "%s: recvfrom(): %s\n",
 			argv[0], strerror(errno));
@@ -171,13 +186,13 @@ main(int argc, char **argv)
 		show(inbuf, nb, nn, hp, sp, ip, bp);
 		*/
 		printf("Got query of %u bytes\n", (unsigned int) nb);
-		status = ldns_wire2pkt(&query_pkt, inbuf, nb);
+		status = ldns_wire2pkt(&query_pkt, inbuf, (size_t) nb);
 		if (status != LDNS_STATUS_OK) {
 			printf("Got bad packet: %s\n", ldns_get_errorstr_by_id(status));
 		} else {
 			ldns_pkt_print(stdout, query_pkt);
 		}
-		
+
 		query_rr = ldns_rr_list_rr(ldns_pkt_question(query_pkt), 0);
 		printf("QUERY RR: \n");
 		ldns_rr_print(stdout, query_rr);
@@ -205,8 +220,20 @@ main(int argc, char **argv)
 		if (status != LDNS_STATUS_OK) {
 			printf("Error creating answer: %s\n", ldns_get_errorstr_by_id(status));
 		} else {
-			nb = (size_t) sendto(sock, outbuf, answer_size, 0, &addr_him, hislen);
+			nb = sendto(sock, outbuf, answer_size, 0, &addr_him, hislen);
 		}
+		
+		ldns_pkt_free(query_pkt);
+		ldns_pkt_free(answer_pkt);
+		LDNS_FREE(outbuf);
+		ldns_rr_list_free(answer_qr);
+		ldns_rr_list_free(answer_an);
+		ldns_rr_list_free(answer_ns);
+		ldns_rr_list_free(answer_ad);
 	}
-        return 0;
+	
+	ldns_rdf_deep_free(origin);
+	ldns_zone_deep_free(zone);
+	
+	return 0;
 }
