@@ -26,9 +26,10 @@ ldns_lookup_table ldns_signing_algorithms[] = {
         { LDNS_SIGN_RSASHA1_NSEC3, "RSASHA1_NSEC3" },
 #ifdef USE_SHA2
         { LDNS_SIGN_RSASHA256, "RSASHA256" },
-        { LDNS_SIGN_RSASHA256_NSEC3, "RSASHA256_NSEC3" },
         { LDNS_SIGN_RSASHA512, "RSASHA512" },
-        { LDNS_SIGN_RSASHA512_NSEC3, "RSASHA512_NSEC3" },
+#endif
+#ifdef USE_GOST
+        { LDNS_SIGN_GOST, "GOST" },
 #endif
         { LDNS_SIGN_DSA, "DSA" },
         { LDNS_SIGN_DSA_NSEC3, "DSA_NSEC3" },
@@ -38,7 +39,6 @@ ldns_lookup_table ldns_signing_algorithms[] = {
         { 0, NULL }
 };
 
-#ifdef HAVE_SSL
 ldns_key_list *
 ldns_key_list_new()
 {
@@ -69,7 +69,9 @@ ldns_key_new()
 		ldns_key_set_inception(newkey, 0);
 		ldns_key_set_expiration(newkey, 0);
 		ldns_key_set_pubkey_owner(newkey, NULL);
+#ifdef HAVE_SSL
 		ldns_key_set_evp_key(newkey, NULL);
+#endif /* HAVE_SSL */
 		ldns_key_set_hmac_key(newkey, NULL);
 		ldns_key_set_external_key(newkey, NULL);
 		return newkey;
@@ -91,7 +93,7 @@ ldns_key_new_frm_engine(ldns_key **key, ENGINE *e, char *key_id, ldns_algorithm
 
 	k = ldns_key_new();
 	k->_key.key = ENGINE_load_private_key(e, key_id, UI_OpenSSL(), NULL);
-	ldns_key_set_algorithm(k, alg);
+	ldns_key_set_algorithm(k, (ldns_signing_algorithm) alg);
 	if (!k->_key.key) {
 		return LDNS_STATUS_ENGINE_KEY_NOT_LOADED;
 	} else {
@@ -101,6 +103,78 @@ ldns_key_new_frm_engine(ldns_key **key, ENGINE *e, char *key_id, ldns_algorithm
 }
 #endif
 
+#ifdef USE_GOST
+int
+ldns_key_EVP_load_gost_id(void)
+{
+	static int gost_id = 0;
+	const EVP_PKEY_ASN1_METHOD* meth;
+	ENGINE* e;
+
+	if(gost_id) return gost_id;
+
+	/* see if configuration loaded gost implementation from other engine*/
+	meth = EVP_PKEY_asn1_find_str(NULL, "gost2001", -1);
+	if(meth) {
+		EVP_PKEY_asn1_get0_info(&gost_id, NULL, NULL, NULL, NULL, meth);
+		return gost_id;
+	}
+
+	/* see if engine can be loaded already */
+	e = ENGINE_by_id("gost");
+	if(!e) {
+		/* load it ourself, in case statically linked */
+		ENGINE_load_builtin_engines();
+		ENGINE_load_dynamic();
+		e = ENGINE_by_id("gost");
+	}
+	if(!e) {
+		/* no gost engine in openssl */
+		return 0;
+	}
+	if(!ENGINE_set_default(e, ENGINE_METHOD_ALL)) {
+		ENGINE_free(e);
+		return 0;
+	}
+
+	meth = EVP_PKEY_asn1_find_str(&e, "gost2001", -1);
+	ENGINE_free(e);
+	if(!meth) {
+		/* algo not found */
+		return 0;
+	}
+	
+	EVP_PKEY_asn1_get0_info(&gost_id, NULL, NULL, NULL, NULL, meth);
+	return gost_id;
+}
+
+/** read GOST private key */
+static EVP_PKEY*
+ldns_key_new_frm_fp_gost_l(FILE* fp, int* line_nr)
+{
+	ssize_t len;
+	char token[16384];
+	const unsigned char* pp;
+	int gost_id;
+	EVP_PKEY* pkey;
+	ldns_rdf* b64rdf = NULL;
+
+	gost_id = ldns_key_EVP_load_gost_id();
+	if(!gost_id)
+		return NULL;
+
+	len = ldns_fget_token_l(fp, token, "", sizeof(token), line_nr);
+	if(len == -1)
+		return NULL;
+	if(ldns_str2rdf_b64(&b64rdf, token) != LDNS_STATUS_OK)
+		return NULL;
+	pp = (unsigned char*)ldns_rdf_data(b64rdf);
+	pkey = d2i_PrivateKey(gost_id, NULL, &pp, ldns_rdf_size(b64rdf));
+	ldns_rdf_deep_free(b64rdf);
+	return pkey;
+}
+#endif
+
 ldns_status
 ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 {
@@ -108,10 +182,12 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 	char *d;
 	ldns_signing_algorithm alg;
 	ldns_rr *key_rr;
+#ifdef HAVE_SSL
 	RSA *rsa;
 	DSA *dsa;
 	unsigned char *hmac;
 	size_t hmac_size;
+#endif /* HAVE_SSL */
 
 	k = ldns_key_new();
 
@@ -174,14 +250,6 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 #else
 		fprintf(stderr, "Warning: SHA256 not compiled into this ");
 		fprintf(stderr, "version of ldns\n");
-#endif
-	}
-	if (strncmp(d, "9 RSASHA256", 2) == 0) {
-#ifdef USE_SHA2
-		alg = LDNS_SIGN_RSASHA256_NSEC3;
-#else
-		fprintf(stderr, "Warning: SHA256 not compiled into this ");
-		fprintf(stderr, "version of ldns\n");
 #endif
 	}
 	if (strncmp(d, "10 RSASHA512", 3) == 0) {
@@ -192,11 +260,11 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 		fprintf(stderr, "version of ldns\n");
 #endif
 	}
-	if (strncmp(d, "11 RSASHA512", 3) == 0) {
-#ifdef USE_SHA2
-		alg = LDNS_SIGN_RSASHA512_NSEC3;
+	if (strncmp(d, "211 GOST", 4) == 0) {
+#ifdef USE_GOST
+		alg = LDNS_SIGN_GOST;
 #else
-		fprintf(stderr, "Warning: SHA512 not compiled into this ");
+		fprintf(stderr, "Warning: GOST not compiled into this ");
 		fprintf(stderr, "version of ldns\n");
 #endif
 	}
@@ -218,36 +286,62 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 		case LDNS_RSASHA1_NSEC3:
 #ifdef USE_SHA2
 		case LDNS_SIGN_RSASHA256:
-		case LDNS_SIGN_RSASHA256_NSEC3:
 		case LDNS_SIGN_RSASHA512:
-		case LDNS_SIGN_RSASHA512_NSEC3:
 #endif
 			ldns_key_set_algorithm(k, alg);
+#ifdef HAVE_SSL
 			rsa = ldns_key_new_frm_fp_rsa_l(fp, line_nr);
+			if (!rsa) {
+				ldns_key_free(k);
+				return LDNS_STATUS_ERR;
+			}
 			ldns_key_set_rsa_key(k, rsa);
 			RSA_free(rsa);
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_SIGN_DSA:
 		case LDNS_DSA_NSEC3:
 			ldns_key_set_algorithm(k, alg);
+#ifdef HAVE_SSL
 			dsa = ldns_key_new_frm_fp_dsa_l(fp, line_nr);
+			if (!dsa) {
+				ldns_key_free(k);
+				return LDNS_STATUS_ERR;
+			}
 			ldns_key_set_dsa_key(k, dsa);
 			DSA_free(dsa);
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_SIGN_HMACMD5:
 		case LDNS_SIGN_HMACSHA1:
 		case LDNS_SIGN_HMACSHA256:
 			ldns_key_set_algorithm(k, alg);
+#ifdef HAVE_SSL
 			hmac = ldns_key_new_frm_fp_hmac_l(fp, line_nr, &hmac_size);
+			if (!hmac) {
+				ldns_key_free(k);
+				return LDNS_STATUS_ERR;
+			}
 			ldns_key_set_hmac_size(k, hmac_size);
 			ldns_key_set_hmac_key(k, hmac);
+#endif /* HAVE_SSL */
+			break;
+		case LDNS_SIGN_GOST:
+			ldns_key_set_algorithm(k, alg);
+#if defined(HAVE_SSL) && defined(USE_GOST)
+			ldns_key_set_evp_key(k, 
+				ldns_key_new_frm_fp_gost_l(fp, line_nr));
+			if(!k->_key.key) {
+				ldns_key_free(k);
+				return LDNS_STATUS_ERR;
+			}
+#endif
 			break;
 		case 0:
 		default:
 			return LDNS_STATUS_SYNTAX_ALG_ERR;
 			break;
 	}
-
 	key_rr = ldns_key2rr(k);
 	ldns_key_set_keytag(k, ldns_calc_keytag(key_rr));
 	ldns_rr_free(key_rr);
@@ -259,6 +353,7 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 	return LDNS_STATUS_ERR;
 }
 
+#ifdef HAVE_SSL
 RSA *
 ldns_key_new_frm_fp_rsa(FILE *f)
 {
@@ -495,7 +590,7 @@ ldns_key_new_frm_fp_hmac(FILE *f, size_t *hmac_size)
 unsigned char *
 ldns_key_new_frm_fp_hmac_l(FILE *f, int *line_nr, size_t *hmac_size)
 {
-	int i;
+	size_t i;
 	char *d;
 	unsigned char *buf;
 
@@ -507,7 +602,9 @@ ldns_key_new_frm_fp_hmac_l(FILE *f, int *line_nr, size_t *hmac_size)
 	if (ldns_fget_keyword_data_l(f, "Key", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
+	i = (size_t) ldns_b64_pton((const char*)d,
+	                           buf,
+	                           ldns_b64_ntop_calculate_size(strlen(d)));
 
 	*hmac_size = i;
 	return buf;
@@ -518,17 +615,52 @@ ldns_key_new_frm_fp_hmac_l(FILE *f, int *line_nr, size_t *hmac_size)
 	*hmac_size = 0;
 	return NULL;
 }
+#endif /* HAVE_SSL */
 
+#ifdef USE_GOST
+static EVP_PKEY*
+ldns_gen_gost_key(void)
+{
+	EVP_PKEY_CTX* ctx;
+	EVP_PKEY* p = NULL;
+	int gost_id = ldns_key_EVP_load_gost_id();
+	if(!gost_id)
+		return NULL;
+	ctx = EVP_PKEY_CTX_new_id(gost_id, NULL);
+	if(!ctx) {
+		/* the id should be available now */
+		return NULL;
+	}
+	if(EVP_PKEY_CTX_ctrl_str(ctx, "paramset", "A") <= 0) {
+		/* cannot set paramset */
+		EVP_PKEY_CTX_free(ctx);
+		return NULL;
+	}
+
+	if(EVP_PKEY_keygen_init(ctx) <= 0) {
+		EVP_PKEY_CTX_free(ctx);
+		return NULL;
+	}
+	if(EVP_PKEY_keygen(ctx, &p) <= 0) {
+		EVP_PKEY_free(p);
+		EVP_PKEY_CTX_free(ctx);
+		return NULL;
+	}
+	EVP_PKEY_CTX_free(ctx);
+	return p;
+}
+#endif
 
 ldns_key *
 ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 {
 	ldns_key *k;
+#ifdef HAVE_SSL
 	DSA *d;
 	RSA *r;
-#ifndef HAVE_SSL
+#else
 	int i;
-	uint16_t offset;
+	uint16_t offset = 0;
 #endif
 	unsigned char *hmac;
 
@@ -541,18 +673,19 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 		case LDNS_SIGN_RSASHA1:
 		case LDNS_SIGN_RSASHA1_NSEC3:
 		case LDNS_SIGN_RSASHA256:
-		case LDNS_SIGN_RSASHA256_NSEC3:
 		case LDNS_SIGN_RSASHA512:
-		case LDNS_SIGN_RSASHA512_NSEC3:
+#ifdef HAVE_SSL
 			r = RSA_generate_key((int)size, RSA_F4, NULL, NULL);
 			if (RSA_check_key(r) != 1) {
 				return NULL;
 			}
 
 			ldns_key_set_rsa_key(k, r);
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_SIGN_DSA:
 		case LDNS_SIGN_DSA_NSEC3:
+#ifdef HAVE_SSL
 			d = DSA_generate_parameters((int)size, NULL, 0, NULL, NULL, NULL, NULL);
 			if (!d) {
 				return NULL;
@@ -561,17 +694,20 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 				return NULL;
 			}
 			ldns_key_set_dsa_key(k, d);
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_SIGN_HMACMD5:
 		case LDNS_SIGN_HMACSHA1:
 		case LDNS_SIGN_HMACSHA256:
+#ifdef HAVE_SSL
 			k->_key.key = NULL;
+#endif /* HAVE_SSL */
 			size = size / 8;
 			ldns_key_set_hmac_size(k, size);
 
 			hmac = LDNS_XMALLOC(unsigned char, size);
 #ifdef HAVE_SSL
-			if (RAND_bytes(hmac, size) != 1) {
+			if (RAND_bytes(hmac, (int) size) != 1) {
 				LDNS_FREE(hmac);
 				ldns_key_free(k);
 				return NULL;
@@ -591,6 +727,11 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 
 			ldns_key_set_flags(k, 0);
 			break;
+		case LDNS_SIGN_GOST:
+#if defined(HAVE_SSL) && defined(USE_GOST)
+			ldns_key_set_evp_key(k, ldns_gen_gost_key());
+#endif /* HAVE_SSL and USE_GOST */
+			break;
 	}
 	ldns_key_set_algorithm(k, alg);
 	return k;
@@ -621,6 +762,7 @@ ldns_key_set_flags(ldns_key *k, uint16_t f)
 	k->_extra.dnssec.flags = f;
 }
 
+#ifdef HAVE_SSL
 void
 ldns_key_set_evp_key(ldns_key *k, EVP_PKEY *e)
 {
@@ -642,6 +784,7 @@ ldns_key_set_dsa_key(ldns_key *k, DSA *d)
 	EVP_PKEY_set1_DSA(key, d);
 	k->_key.key  = key;
 }
+#endif /* HAVE_SSL */
 
 void
 ldns_key_set_hmac_key(ldns_key *k, unsigned char *hmac)
@@ -731,6 +874,7 @@ ldns_key_use(const ldns_key *k)
 	return false;
 }
 
+#ifdef HAVE_SSL
 EVP_PKEY *
 ldns_key_evp_key(const ldns_key *k)
 {
@@ -756,6 +900,7 @@ ldns_key_dsa_key(const ldns_key *k)
 		return NULL;
 	}
 }
+#endif /* HAVE_SSL */
 
 unsigned char *
 ldns_key_hmac_key(const ldns_key *k)
@@ -886,6 +1031,7 @@ ldns_key_list_pop_key(ldns_key_list *key_list)
         return pop;
 }       
 
+#ifdef HAVE_SSL
 static bool
 ldns_key_rsa2bin(unsigned char *data, RSA *k, uint16_t *size)
 {
@@ -947,6 +1093,27 @@ ldns_key_dsa2bin(unsigned char *data, DSA *k, uint16_t *size)
 	return true;
 }
 
+#ifdef USE_GOST
+static bool
+ldns_key_gost2bin(unsigned char* data, EVP_PKEY* k, uint16_t* size)
+{
+	int i;
+	unsigned char* pp = NULL;
+	if(i2d_PUBKEY(k, &pp) != 37 + 64) {
+		/* expect 37 byte(ASN header) and 64 byte(X and Y) */
+		CRYPTO_free(pp);
+		return false;
+	}
+	/* omit ASN header */
+	for(i=0; i<64; i++)
+		data[i] = pp[i+37];
+	CRYPTO_free(pp);
+	*size = 64;
+	return true;
+}
+#endif /* USE_GOST */
+#endif /* HAVE_SSL */
+
 ldns_rr *
 ldns_key2rr(const ldns_key *k)
 {
@@ -959,8 +1126,10 @@ ldns_key2rr(const ldns_key *k)
 	ldns_rdf *keybin;
 	unsigned char *bin = NULL;
 	uint16_t size = 0;
+#ifdef HAVE_SSL
 	RSA *rsa = NULL;
 	DSA *dsa = NULL;
+#endif /* HAVE_SSL */
 	int internal_data = 0;
 
 	pubkey = ldns_rr_new();
@@ -996,11 +1165,10 @@ ldns_key2rr(const ldns_key *k)
 		case LDNS_RSASHA1:
 		case LDNS_RSASHA1_NSEC3:
 		case LDNS_RSASHA256:
-		case LDNS_RSASHA256_NSEC3:
 		case LDNS_RSASHA512:
-		case LDNS_RSASHA512_NSEC3:
 			ldns_rr_push_rdf(pubkey,
 						  ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, ldns_key_algorithm(k)));
+#ifdef HAVE_SSL
 			rsa =  ldns_key_rsa_key(k);
 			if (rsa) {
 				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
@@ -1013,11 +1181,13 @@ ldns_key2rr(const ldns_key *k)
 				RSA_free(rsa);
 				internal_data = 1;
 			}
+#endif
 			size++;
 			break;
 		case LDNS_SIGN_DSA:
 			ldns_rr_push_rdf(pubkey,
 					ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, LDNS_DSA));
+#ifdef HAVE_SSL
 			dsa = ldns_key_dsa_key(k);
 			if (dsa) {
 				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
@@ -1030,10 +1200,12 @@ ldns_key2rr(const ldns_key *k)
 				DSA_free(dsa);
 				internal_data = 1;
 			}
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_DSA_NSEC3:
 			ldns_rr_push_rdf(pubkey,
 					ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, LDNS_DSA_NSEC3));
+#ifdef HAVE_SSL
 			dsa = ldns_key_dsa_key(k);
 			if (dsa) {
 				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
@@ -1046,11 +1218,25 @@ ldns_key2rr(const ldns_key *k)
 				DSA_free(dsa);
 				internal_data = 1;
 			}
+#endif /* HAVE_SSL */
 			break;
+		case LDNS_SIGN_GOST:
+			ldns_rr_push_rdf(pubkey, ldns_native2rdf_int8(
+				LDNS_RDF_TYPE_ALG, ldns_key_algorithm(k)));
+#if defined(HAVE_SSL) && defined(USE_GOST)
+			bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
+			if (!bin) 
+				return NULL;
+			if (!ldns_key_gost2bin(bin, k->_key.key, &size)) {
+				return NULL;
+			}
+			internal_data = 1;
+			break;
+#endif /* HAVE_SSL and USE_GOST */
 		case LDNS_SIGN_HMACMD5:
 		case LDNS_SIGN_HMACSHA1:
 		case LDNS_SIGN_HMACSHA256:
-			bin = LDNS_XMALLOC(unsigned char, size);
+			bin = LDNS_XMALLOC(unsigned char, ldns_key_hmac_size(k));
 			if (!bin) {
 				return NULL;
 			}
@@ -1083,9 +1269,11 @@ ldns_key_deep_free(ldns_key *key)
 	if (ldns_key_pubkey_owner(key)) {
 		ldns_rdf_deep_free(ldns_key_pubkey_owner(key));
 	}
+#ifdef HAVE_SSL
 	if (ldns_key_evp_key(key)) {
 		EVP_PKEY_free(ldns_key_evp_key(key));
 	}
+#endif /* HAVE_SSL */
 	if (ldns_key_hmac_key(key)) {
 		free(ldns_key_hmac_key(key));
 	}
@@ -1154,7 +1342,7 @@ ldns_key_get_file_base_name(ldns_key *key)
 	
 	buffer = ldns_buffer_new(255);
 	ldns_buffer_printf(buffer, "K");
-	ldns_rdf2buffer_str_dname(buffer, ldns_key_pubkey_owner(key));
+	(void)ldns_rdf2buffer_str_dname(buffer, ldns_key_pubkey_owner(key));
 	ldns_buffer_printf(buffer,
 	                   "+%03u+%05u",
 			   ldns_key_algorithm(key),
@@ -1164,4 +1352,13 @@ ldns_key_get_file_base_name(ldns_key *key)
 	return file_base_name;
 }
 
-#endif /* HAVE_SSL */
+int ldns_key_algo_supported(int algo)
+{
+	ldns_lookup_table *lt = ldns_signing_algorithms;
+	while(lt->name) {
+		if(lt->id == algo)
+			return 1;
+		lt++;
+	}
+	return 0;
+}
