@@ -32,6 +32,10 @@
 /** The bytes TTL, CLASS and length use up in an rr */
 #define LDNS_RR_OVERHEAD	10
 
+/* The first fields are 'common' and can be referenced instantly */
+#define LDNS_RDATA_FIELD_DESCRIPTORS_COMMON 51
+
+
 
 /**
  *  The different RR classes.
@@ -44,8 +48,8 @@ enum ldns_enum_rr_class
 	LDNS_RR_CLASS_CH	= 3,
 	/** Hesiod (Dyer 87) */
 	LDNS_RR_CLASS_HS	= 4,
-        /** None class, dynmaic update */
-        LDNS_RR_CLASS_NONE      = 254,
+    /** None class, dynamic update */
+    LDNS_RR_CLASS_NONE      = 254,
 	/** Any class */
 	LDNS_RR_CLASS_ANY	= 255,
 
@@ -165,6 +169,13 @@ enum ldns_enum_rr_type
 	LDNS_RR_TYPE_RRSIG = 46,
 	LDNS_RR_TYPE_NSEC = 47,      
 	LDNS_RR_TYPE_DNSKEY = 48,
+	LDNS_RR_TYPE_DHCID = 49,
+
+        /* TODO: Temporary assume type code 50, 51 (from draft-11) */
+	LDNS_RR_TYPE_NSEC3 = 50,
+	LDNS_RR_TYPE_NSEC3PARAMS = 51,
+
+	LDNS_RR_TYPE_SPF = 99,
 
 	LDNS_RR_TYPE_UINFO = 100,
 	LDNS_RR_TYPE_UID = 101,
@@ -181,6 +192,12 @@ enum ldns_enum_rr_type
 	/**  any type (wildcard) */
 	LDNS_RR_TYPE_ANY = 255,
 
+	/* RFC 4431, 5074, DNSSEC Lookaside Validation */
+	LDNS_RR_TYPE_DLV = 32769,
+
+	/* type codes from nsec3 experimental phase
+	LDNS_RR_TYPE_NSEC3 = 65324,
+	LDNS_RR_TYPE_NSEC3PARAMS = 65325, */
 	LDNS_RR_TYPE_FIRST = 0,
 	LDNS_RR_TYPE_LAST  = 65535,
 	LDNS_RR_TYPE_COUNT = LDNS_RR_TYPE_LAST - LDNS_RR_TYPE_FIRST + 1
@@ -301,6 +318,8 @@ struct ldns_struct_rr_descriptor
 	ldns_rdf_type _variable;
 	/** Specifies whether compression can be used for dnames in this RR type. */
 	ldns_rr_compress _compress;
+	/** The number of DNAMEs in the _wireformat string, for parsing. */
+	uint8_t _dname_count;
 };
 typedef struct ldns_struct_rr_descriptor ldns_rr_descriptor;
 
@@ -326,17 +345,38 @@ void ldns_rr_free(ldns_rr *rr);
 /**
  * creates an rr from a string.
  * The string should be a fully filled-in rr, like
- * ownername &lt;space&gt; TTL &lt;space&gt; CLASS &lt;space&gt; TYPE &lt;space&gt; RDATA.
+ * ownername &lt;space&gt; TTL &lt;space&gt; CLASS &lt;space&gt; 
+ * TYPE &lt;space&gt; RDATA.
+ * \param[out] n the rr to return
+ * \param[in] str the string to convert
+ * \param[in] default_ttl default ttl value for the rr.
+ *            If 0 DEF_TTL will be used
+ * \param[in] origin when the owner is relative add this. 
+ *	The caller must ldns_rdf_deep_free it.
+ * \param[out] prev the previous ownername. if this value is not NULL,
+ * the function overwrites this with the ownername found in this
+ * string. The caller must then ldns_rdf_deep_free it.
+ * \return a status msg describing an error or LDNS_STATUS_OK
+ */
+ldns_status ldns_rr_new_frm_str(ldns_rr **n, const char *str,
+                                uint32_t default_ttl, ldns_rdf *origin,
+                                ldns_rdf **prev);
+
+/**
+ * creates an rr for the question section from a string, i.e.
+ * without RDATA fields
+ * Origin and previous RR functionality are the same as in
+ * ldns_rr_new_frm_str()
  * \param[out] n the rr to return
  * \param[in] str the string to convert
- * \param[in] default_ttl pointer to a default ttl for the rr. If 0 DEF_TTL will be used
  * \param[in] origin when the owner is relative add this. 
  *	The caller must ldns_rdf_deep_free it.
  * \param prev the previous ownername. the function overwrite this with
  * the current found ownername. The caller must ldns_rdf_deep_free it.
  * \return a status msg describing an error or LDNS_STATUS_OK
  */
-ldns_status ldns_rr_new_frm_str(ldns_rr **n, const char *str, uint16_t default_ttl, ldns_rdf *origin, ldns_rdf **prev);
+ldns_status ldns_rr_new_question_frm_str(ldns_rr **n, const char *str,
+                                ldns_rdf *origin, ldns_rdf **prev);
 
 /**
  * creates a new rr from a file containing a string.
@@ -352,25 +392,24 @@ ldns_status ldns_rr_new_frm_str(ldns_rr **n, const char *str, uint16_t default_t
  *	      The caller must ldns_rdf_deep_free it.
  * \return a ldns_status with an error or LDNS_STATUS_OK
  */
-ldns_status ldns_rr_new_frm_fp(ldns_rr **rr, FILE *fp, uint16_t *default_ttl, ldns_rdf **origin, ldns_rdf **prev);
+ldns_status ldns_rr_new_frm_fp(ldns_rr **rr, FILE *fp, uint32_t *default_ttl, ldns_rdf **origin, ldns_rdf **prev);
 
 /**
  * creates a new rr from a file containing a string.
  * \param[out] rr the new rr
  * \param[in] fp the file pointer to use
- * \param[in] fp the file pointer to use
- * \param[in] default_ttl a default ttl for the rr. If 0 DEF_TTL will be used
+ * \param[in] default_ttl a default ttl for the rr. If NULL DEF_TTL will be used
  *            the pointer will be updated if the file contains a $TTL directive
- *	      The caller must ldns_rdf_deep_free it.
  * \param[in] origin when the owner is relative add this
  * 	      the pointer will be updated if the file contains a $ORIGIN directive
  *	      The caller must ldns_rdf_deep_free it.
  * \param[in] line_nr pointer to an integer containing the current line number (for debugging purposes)
  * \param[in] prev when the owner is whitespaces use this as the * ownername
  *            the pointer will be updated after the call
+ *	      The caller must ldns_rdf_deep_free it.
  * \return a ldns_status with an error or LDNS_STATUS_OK
  */
-ldns_status ldns_rr_new_frm_fp_l(ldns_rr **rr, FILE *fp, uint16_t *default_ttl, ldns_rdf **origin, ldns_rdf **prev, int *line_nr);
+ldns_status ldns_rr_new_frm_fp_l(ldns_rr **rr, FILE *fp, uint32_t *default_ttl, ldns_rdf **origin, ldns_rdf **prev, int *line_nr);
 
 /**
  * sets the owner in the rr structure.
@@ -591,7 +630,7 @@ ldns_rr_list* ldns_rr_list_pop_rr_list(ldns_rr_list *rr_list, size_t size);
  * \param[in] rr the rr to check
  * \return true if rr_list contains rr, false otherwise
  */
-bool ldns_rr_list_contains_rr(ldns_rr_list *rr_list, ldns_rr *rr); 
+bool ldns_rr_list_contains_rr(const ldns_rr_list *rr_list, ldns_rr *rr); 
 
 /**
  * checks if an rr_list is a rrset.
@@ -651,7 +690,7 @@ ldns_rr* ldns_rr_clone(const ldns_rr *rr);
  * \param[in] rrlist the rrlist to clone
  * \return the cloned rr list
  */
-ldns_rr_list* ldns_rr_list_clone(ldns_rr_list *rrlist);
+ldns_rr_list* ldns_rr_list_clone(const ldns_rr_list *rrlist);
 
 /**
  * sorts an rr_list (canonical wire format). the sorting is done inband.
@@ -661,7 +700,7 @@ ldns_rr_list* ldns_rr_list_clone(ldns_rr_list *rrlist);
 void ldns_rr_list_sort(ldns_rr_list *unsorted);
 
 /**
- * compares two rrs.
+ * compares two rrs. The TTL is not looked at.
  * \param[in] rr1 the first one
  * \param[in] rr2 the second one
  * \return 0 if equal
@@ -669,6 +708,7 @@ void ldns_rr_list_sort(ldns_rr_list *unsorted);
  *         +1 if rr2 comes before rr1
  */
 int ldns_rr_compare(const ldns_rr *rr1, const ldns_rr *rr2);
+int ldns_rr_compare_nsec3(const ldns_rr *rr1, const ldns_rr *rr2);
 
 /**
  * compares two rrs, up to the rdata.
@@ -794,4 +834,24 @@ ldns_rr_list *ldns_rr_list_subtype_by_rdf(ldns_rr_list *l, ldns_rdf *r, size_t p
  */
 ldns_rr_type    ldns_rdf2rr_type(const ldns_rdf *rd);
 
+/**
+ * Returns the type of the first element of the RR
+ * If there are no elements present, 0 is returned
+ * 
+ * \param[in] rr_list The rr list
+ * \return rr_type of the first element, or 0 if the list is empty
+ */
+ldns_rr_type
+ldns_rr_list_type(const ldns_rr_list *rr_list);
+
+/**
+ * Returns the owner domain name rdf of the first element of the RR
+ * If there are no elements present, NULL is returned
+ * 
+ * \param[in] rr_list The rr list
+ * \return dname of the first element, or NULL if the list is empty
+ */
+ldns_rdf *
+ldns_rr_list_owner(const ldns_rr_list *rr_list);
+
 #endif /* LDNS_RR_H */
