@@ -145,7 +145,7 @@ ldns_pkt_additional(const ldns_pkt *packet)
 
 /* return ALL section concatenated */
 ldns_rr_list *
-ldns_pkt_all(ldns_pkt *packet)
+ldns_pkt_all(const ldns_pkt *packet)
 {
 	ldns_rr_list *all, *prev_all;
 
@@ -164,7 +164,7 @@ ldns_pkt_all(ldns_pkt *packet)
 }
 
 ldns_rr_list *
-ldns_pkt_all_noquestion(ldns_pkt *packet)
+ldns_pkt_all_noquestion(const ldns_pkt *packet)
 {
 	ldns_rr_list *all, *all2;
 
@@ -238,7 +238,7 @@ ldns_pkt_set_edns_do(ldns_pkt *packet, bool value)
 	if (value) {
 		packet->_edns_z = packet->_edns_z | LDNS_EDNS_MASK_DO_BIT;
 	} else {
-		packet->_edns_z = packet->_edns_z & !LDNS_EDNS_MASK_DO_BIT;
+		packet->_edns_z = packet->_edns_z & ~LDNS_EDNS_MASK_DO_BIT;
 	}
 }
 
@@ -250,7 +250,9 @@ ldns_pkt_edns_data(const ldns_pkt *packet)
 
 /* return only those rr that share the ownername */
 ldns_rr_list *
-ldns_pkt_rr_list_by_name(ldns_pkt *packet, ldns_rdf *ownername, ldns_pkt_section sec)
+ldns_pkt_rr_list_by_name(ldns_pkt *packet,
+                         ldns_rdf *ownername,
+                         ldns_pkt_section sec)
 {
 	ldns_rr_list *rrs;
 	ldns_rr_list *new;
@@ -279,7 +281,9 @@ ldns_pkt_rr_list_by_name(ldns_pkt *packet, ldns_rdf *ownername, ldns_pkt_section
 
 /* return only those rr that share a type */
 ldns_rr_list *
-ldns_pkt_rr_list_by_type(ldns_pkt *packet, ldns_rr_type type, ldns_pkt_section sec)
+ldns_pkt_rr_list_by_type(const ldns_pkt *packet,
+                         ldns_rr_type type,
+                         ldns_pkt_section sec)
 {
 	ldns_rr_list *rrs;
 	ldns_rr_list *new;
@@ -313,7 +317,10 @@ ldns_pkt_rr_list_by_type(ldns_pkt *packet, ldns_rr_type type, ldns_pkt_section s
 
 /* return only those rrs that share name and type */
 ldns_rr_list *
-ldns_pkt_rr_list_by_name_and_type(ldns_pkt *packet, ldns_rdf *ownername, ldns_rr_type type, ldns_pkt_section sec)
+ldns_pkt_rr_list_by_name_and_type(const ldns_pkt *packet,
+                                  const ldns_rdf *ownername,
+                                  ldns_rr_type type,
+                                  ldns_pkt_section sec)
 {
 	ldns_rr_list *rrs;
 	ldns_rr_list *new;
@@ -349,24 +356,27 @@ ldns_pkt_rr_list_by_name_and_type(ldns_pkt *packet, ldns_rdf *ownername, ldns_rr
 bool
 ldns_pkt_rr(ldns_pkt *pkt, ldns_pkt_section sec, ldns_rr *rr)
 {
-	ldns_rr_list *rrs;
-	uint16_t rr_count;
-	uint16_t i;
+	bool result = false;
 
-	rrs = ldns_pkt_get_section_clone(pkt, sec);
-	if (!rrs) {
-		return false;
+	switch (sec) {
+	case LDNS_SECTION_QUESTION:
+		return ldns_rr_list_contains_rr(ldns_pkt_question(pkt), rr);
+	case LDNS_SECTION_ANSWER:
+		return ldns_rr_list_contains_rr(ldns_pkt_answer(pkt), rr);
+	case LDNS_SECTION_AUTHORITY:
+		return ldns_rr_list_contains_rr(ldns_pkt_authority(pkt), rr);
+	case LDNS_SECTION_ADDITIONAL:
+		return ldns_rr_list_contains_rr(ldns_pkt_additional(pkt), rr);
+	case LDNS_SECTION_ANY:
+		result = ldns_rr_list_contains_rr(ldns_pkt_question(pkt), rr);
+	case LDNS_SECTION_ANY_NOQUESTION:
+		result = result
+		    || ldns_rr_list_contains_rr(ldns_pkt_answer(pkt), rr)
+		    || ldns_rr_list_contains_rr(ldns_pkt_authority(pkt), rr)
+		    || ldns_rr_list_contains_rr(ldns_pkt_additional(pkt), rr);
 	}
-	rr_count = ldns_rr_list_rr_count(rrs);
-	
-	/* walk the rrs and compare them with rr */	
-	for(i = 0; i < rr_count; i++) {
-		if (ldns_rr_compare(ldns_rr_list_rr(rrs, i), rr) == 0) {
-			/* a match */
-			return true;
-		}
-	}
-	return false;
+
+	return result;
 }
 
 uint16_t
@@ -409,7 +419,7 @@ ldns_pkt_empty(ldns_pkt *p)
 
 
 ldns_rr_list *
-ldns_pkt_get_section_clone(ldns_pkt *packet, ldns_pkt_section s)
+ldns_pkt_get_section_clone(const ldns_pkt *packet, ldns_pkt_section s)
 {
 	switch(s) {
 	case LDNS_SECTION_QUESTION:
@@ -713,7 +723,8 @@ bool
 ldns_pkt_edns(const ldns_pkt *pkt) {
 	return (ldns_pkt_edns_udp_size(pkt) > 0 ||
 		ldns_pkt_edns_extended_rcode(pkt) > 0 ||
-		ldns_pkt_edns_data(pkt)
+		ldns_pkt_edns_data(pkt) ||
+		ldns_pkt_edns_do(pkt)
 	       );
 }
 
@@ -982,7 +993,16 @@ ldns_pkt_clone(ldns_pkt *pkt)
 	ldns_pkt_set_size(new_pkt, ldns_pkt_size(pkt));
 	ldns_pkt_set_tsig(new_pkt, ldns_pkt_tsig(pkt));
 	
-	/* todo: edns? jelte?? */
+	ldns_pkt_set_edns_udp_size(new_pkt, ldns_pkt_edns_udp_size(pkt));
+	ldns_pkt_set_edns_extended_rcode(new_pkt, 
+		ldns_pkt_edns_extended_rcode(pkt));
+	ldns_pkt_set_edns_version(new_pkt, ldns_pkt_edns_version(pkt));
+	ldns_pkt_set_edns_z(new_pkt, ldns_pkt_edns_z(pkt));
+	if(ldns_pkt_edns_data(pkt))
+		ldns_pkt_set_edns_data(new_pkt, 
+			ldns_rdf_clone(ldns_pkt_edns_data(pkt)));
+	ldns_pkt_set_edns_do(new_pkt, ldns_pkt_edns_do(pkt));
+
 	ldns_rr_list_deep_free(new_pkt->_question);
 	ldns_rr_list_deep_free(new_pkt->_answer);
 	ldns_rr_list_deep_free(new_pkt->_authority);
