@@ -432,7 +432,8 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 							ldns_write_uint16(hex_data, hex_data_size);
 							ldns_hexstring_to_data(hex_data + 2, hex_data_str);
 							hex_pos = 0;
-							ldns_wire2rdf(new, hex_data, hex_data_size+2, &hex_pos);
+							(void) ldns_wire2rdf(new, hex_data,
+							                 hex_data_size+2, &hex_pos);
 							LDNS_FREE(hex_data);
 						} else {
 							r = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_HEX, hex_data_str);
@@ -467,7 +468,8 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 									rd);
 
 							/* check if the origin should be used or concatenated */
-							if (rd_strlen == 1 && ldns_rdf_data(r)[1] == '@') {
+							if (ldns_rdf_size(r) > 1 && ldns_rdf_data(r)[0] == 1 
+								&& ldns_rdf_data(r)[1] == '@') {
 								ldns_rdf_deep_free(r);
 								if (origin) {
 									r = ldns_rdf_clone(origin);
@@ -479,7 +481,7 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 										r = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_DNAME, ".");
 									}
 								}
-							} else if (rd_strlen > 1 && !ldns_dname_str_absolute(rd) && origin) {
+							} else if (rd_strlen >= 1 && !ldns_dname_str_absolute(rd) && origin) {
 								if (ldns_dname_cat(r, origin) != LDNS_STATUS_OK) {
 									return LDNS_STATUS_ERR;
 								}
@@ -934,6 +936,7 @@ ldns_rr_list_subtype_by_rdf(ldns_rr_list *l, ldns_rdf *r, size_t pos)
 			pos);
 		if (!list_rdf) {
 			/* pos is too large or any other error */
+			ldns_rr_list_deep_free(subtyped);
 			return NULL;
 		}
 
@@ -947,6 +950,7 @@ ldns_rr_list_subtype_by_rdf(ldns_rr_list *l, ldns_rdf *r, size_t pos)
 	if (ldns_rr_list_rr_count(subtyped) > 0) {
 		return subtyped;
 	} else {
+		ldns_rr_list_free(subtyped);
 		return NULL;
 	}
 }
@@ -1081,7 +1085,7 @@ ldns_is_rrset(ldns_rr_list *rr_list)
 	ldns_rr *tmp;
 	size_t i;
 	
-	if (!rr_list) {
+	if (!rr_list || ldns_rr_list_rr_count(rr_list) == 0) {
 		return false;
 	}
 
@@ -1093,7 +1097,7 @@ ldns_is_rrset(ldns_rr_list *rr_list)
 
 	/* compare these with the rest of the rr_list, start with 1 */
 	for (i = 1; i < ldns_rr_list_rr_count(rr_list); i++) {
-		tmp = ldns_rr_list_rr(rr_list, 1);
+		tmp = ldns_rr_list_rr(rr_list, i);
 		if (t != ldns_rr_get_type(tmp)) {
 			return false;
 		}
@@ -1264,7 +1268,7 @@ ldns_rr_list_clone(const ldns_rr_list *rrlist)
 		    );
 		if (!r) {
 			/* huh, failure in cloning */
-			ldns_rr_list_free(new_list);
+			ldns_rr_list_deep_free(new_list);
 			return NULL;
 		}
 		ldns_rr_list_push_rr(new_list, r);
@@ -1494,13 +1498,36 @@ ldns_rr_compare(const ldns_rr *rr1, const ldns_rr *rr2)
 	return result;
 }
 
+/* convert dnskey to a ds with the given algorithm,
+ * then compare the result with the given ds */
+static int
+ldns_rr_compare_ds_dnskey(ldns_rr *ds,
+                          ldns_rr *dnskey)
+{
+	ldns_rr *ds_gen;
+	bool result = false;
+	ldns_algorithm algo;
+	
+	if (!dnskey || !ds ||
+	    ldns_rr_get_type(ds) != LDNS_RR_TYPE_DS ||
+	    ldns_rr_get_type(dnskey) != LDNS_RR_TYPE_DNSKEY) {
+		return false;
+	}
+	
+	algo = ldns_rdf2native_int8(ldns_rr_rdf(ds, 2));
+
+	ds_gen = ldns_key_rr2ds(dnskey, algo);
+	if (ds_gen) {
+		result = ldns_rr_compare(ds, ds_gen) == 0;
+		ldns_rr_free(ds_gen);
+	}
+	return result;
+}
+
 bool
 ldns_rr_compare_ds(const ldns_rr *orr1, const ldns_rr *orr2)
 {
 	bool result;
-#ifdef HAVE_SSL
-	ldns_rr *ds_repr;
-#endif /* HAVE_SSL */
 	ldns_rr *rr1 = ldns_rr_clone(orr1);
 	ldns_rr *rr2 = ldns_rr_clone(orr2);
 	
@@ -1508,24 +1535,15 @@ ldns_rr_compare_ds(const ldns_rr *orr1, const ldns_rr *orr2)
 	ldns_rr_set_ttl(rr1, 0);
 	ldns_rr_set_ttl(rr2, 0);
 
-#ifdef HAVE_SSL
 	if (ldns_rr_get_type(rr1) == LDNS_RR_TYPE_DS &&
 	    ldns_rr_get_type(rr2) == LDNS_RR_TYPE_DNSKEY) {
-	    	ds_repr = ldns_key_rr2ds(rr2, LDNS_SHA1);
-	    	result = (ldns_rr_compare(rr1, ds_repr) == 0);
-	    	ldns_rr_free(ds_repr);
+		result = ldns_rr_compare_ds_dnskey(rr1, rr2);
 	} else if (ldns_rr_get_type(rr1) == LDNS_RR_TYPE_DNSKEY &&
 	    ldns_rr_get_type(rr2) == LDNS_RR_TYPE_DS) {
-	    	ds_repr = ldns_key_rr2ds(rr1, LDNS_SHA1);
-	    	result = (ldns_rr_compare(rr2, ds_repr) == 0);
-
-	    	ldns_rr_free(ds_repr);
+		result = ldns_rr_compare_ds_dnskey(rr2, rr1);
 	} else {
 		result = (ldns_rr_compare(rr1, rr2) == 0);
 	}	
-#else
-		result = (ldns_rr_compare(rr1, rr2) == 0);
-#endif /* HAVE_SSL */
 	
 	ldns_rr_free(rr1);
 	ldns_rr_free(rr2);
@@ -1734,7 +1752,7 @@ static const ldns_rdf_type type_kx_wireformat[] = {
 static const ldns_rdf_type type_cert_wireformat[] = {
 	 LDNS_RDF_TYPE_CERT_ALG, LDNS_RDF_TYPE_INT16, LDNS_RDF_TYPE_ALG, LDNS_RDF_TYPE_B64
 };
-static const ldns_rdf_type type_a6_wireformat[] = { LDNS_RDF_TYPE_DNAME };
+static const ldns_rdf_type type_a6_wireformat[] = { LDNS_RDF_TYPE_UNKNOWN };
 static const ldns_rdf_type type_dname_wireformat[] = { LDNS_RDF_TYPE_DNAME };
 static const ldns_rdf_type type_sink_wireformat[] = { LDNS_RDF_TYPE_INT8,
 	LDNS_RDF_TYPE_INT8, LDNS_RDF_TYPE_INT8, LDNS_RDF_TYPE_B64
@@ -1875,7 +1893,7 @@ static ldns_rr_descriptor rdata_field_descriptors[] = {
 	/* 37 */
 	{LDNS_RR_TYPE_CERT, "CERT", 4, 4, type_cert_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 	/* 38 */
-	{LDNS_RR_TYPE_A6, "A6", 1, 1, type_a6_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 1 },
+	{LDNS_RR_TYPE_A6, "A6", 1, 1, type_a6_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 	/* 39 */
 	{LDNS_RR_TYPE_DNAME, "DNAME", 1, 1, type_dname_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 1 },
 	/* 40 */
@@ -1950,7 +1968,7 @@ static ldns_rr_descriptor rdata_field_descriptors[] = {
 {LDNS_RR_TYPE_NULL, "TYPE96", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE97", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE98", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
-{LDNS_RR_TYPE_NULL, "TYPE99", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
+{LDNS_RR_TYPE_SPF,  "SPF", 1, 0, NULL, LDNS_RDF_TYPE_STR, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE100", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE101", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE102", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
