@@ -389,8 +389,7 @@ ldns_str2rdf_str(ldns_rdf **rd, const char *str)
 {
 	uint8_t *data;
 	size_t i, str_i;
-	int esc;
-	
+
 	if (strlen(str) > 255) {
 		return LDNS_STATUS_INVALID_STR;
 	}
@@ -398,20 +397,8 @@ ldns_str2rdf_str(ldns_rdf **rd, const char *str)
 	data = LDNS_XMALLOC(uint8_t, strlen(str) + 1);
 	i = 1;
 	for (str_i = 0; str_i < strlen(str); str_i++) {
-		if (str[str_i] == '\\') {
-			esc = parse_escape((uint8_t *) (str + str_i), data + i);
-			if (esc > 0) {
-				str_i += esc;
-				i++;
-			} else {
-				LDNS_FREE(data);
-				*rd = NULL;
-				return LDNS_STATUS_SYNTAX_BAD_ESCAPE;
-			}
-		} else {
-			data[i] = (uint8_t) str[str_i];
-			i++;
-		}
+		data[i] = (uint8_t) str[str_i];
+		i++;
 	}
 	data[0] = i - 1;
 	*rd = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_STR, i, data);
